#!/usr/bin/env nix-shell
#!nix-shell deps.nix -i python

import yaml
from pcbnew import *


PCB_FILE  = '../CyanStripe/CyanStripe.kicad_pcb'
SCAD_FILE = 'GENERATED.scad'





def genericRectGen(module):
  b = module.GetBoundingBox()

  shrink = 0.95

  w = ToMM(b.GetWidth()) * shrink
  h = ToMM(b.GetHeight()) * shrink

  return f'roundRect({w}, {h}, rpct=10, $fs=0.05, $fa=0.05);'

def genericCircleGen(module):
  b = module.GetBoundingBox()

  shrink = 0.95

  wh = ToMM( (b.GetWidth() + b.GetHeight()) / 2 ) * shrink

  return f'circle(d={wh}, $fs=0.05, $fa=0.05);'


eligibleForCircle = [ 'M' ]
eligibleForRect = [ 'R', 'C', 'U', 'J', 'D', 'L' ]

def applyGenerics(module):
  ref = str(module.GetReference())

  def _eligible(_list):
    for _ref in _list:
      if ref.startswith(_ref):
        return True
    return False

  if _eligible(eligibleForCircle):
    return genericCircleGen(module)
  elif _eligible(eligibleForRect):
    return genericRectGen(module)


with open('footprints.yml', 'r') as f:
  footprints = yaml.safe_load(f.read())

def searchFootprint(module):

  fpid    = module.GetFPID()
  name    = str(fpid.GetLibItemName())
  lib     = str(fpid.GetLibNickname())

  rot     = module.GetOrientationDegrees()
  rotCode = f'rotate([0, 0, {rot}])'

  code = None

  for fp in footprints:
    if 'type' in fp and fp['type'] == name:
      if 'lib' in fp:
        if fp['lib'] == lib:
          code = fp['code']
      else:
        code = fp['code']
  
  if code is not None:
    return f'{rotCode} {code}'
  else:
    return applyGenerics(module)


class SCAD:
  def __init__(self, filename):
    self.__filename = filename
    self.content = ''

  def print(self, any):
    self.content = self.content + f'{any}\n'

  def save(self):
    with open(self.__filename, 'w') as f:
      f.write(self.content)


pcb = LoadBoard(PCB_FILE)
scad = SCAD(SCAD_FILE)

scad.print('')
scad.print('$fs = 0.8;')
scad.print('$fa = 0.5;')
scad.print('\n')
scad.print('use <footprints.scad>;')
scad.print('\n\n')


for module in pcb.GetModules():
  ref = str(module.GetReference())

  fpid    = module.GetFPID()
  name    = str(fpid.GetLibItemName())
  library = str(fpid.GetLibNickname())

  pos     = module.GetPosition()
  x       = ToMM(pos.x)
  y       = ToMM(pos.y)


  code = searchFootprint(module)

  print(f'Module: {ref}   type={name}  library={library}   code={code}')

  if code is not None:
    scad.print(f'// {ref}  type={name}  lib={library}')
    scad.print(f'translate([{x}  -100, -{y}  +50, 0]) {code}')
    scad.print('')


scad.save()
